"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var differenceWith_1 = require("./differenceWith");
var equals_1 = require("./equals");
var pipe_1 = require("./pipe");
var take_1 = require("./take");
var _counter_1 = require("./_counter");
var source = [{ a: 1 }, { a: 2 }, { a: 3 }, { a: 4 }];
var other = [{ a: 2 }, { a: 5 }, { a: 3 }];
var expected = [{ a: 1 }, { a: 4 }];
describe('data_first', function () {
    test('should return difference', function () {
        expect((0, differenceWith_1.differenceWith)(source, other, equals_1.equals)).toEqual(expected);
    });
    test('should allow differencing different data types', function () {
        expect((0, differenceWith_1.differenceWith)([1, 2, 3, 4], ['2', '3'], function (a, b) { return a.toString() === b; })).toEqual([1, 4]);
    });
});
describe('data_last', function () {
    test('should return difference', function () {
        expect((0, differenceWith_1.differenceWith)(other, equals_1.equals)(source)).toEqual(expected);
    });
    test('should allow differencing different data types', function () {
        expect((0, pipe_1.pipe)([1, 2, 3, 4], (0, differenceWith_1.differenceWith)(['2', '3'], function (a, b) { return a.toString() === b; }))).toEqual([1, 4]);
    });
    test('lazy', function () {
        var counter = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([{ a: 1 }, { a: 2 }, { a: 3 }, { a: 4 }, { a: 5 }, { a: 6 }], counter.fn(), (0, differenceWith_1.differenceWith)([{ a: 2 }, { a: 3 }], equals_1.equals), (0, take_1.take)(2));
        expect(counter.count).toHaveBeenCalledTimes(4);
        expect(result).toEqual([{ a: 1 }, { a: 4 }]);
    });
});
