"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var first_1 = require("./first");
var pipe_1 = require("./pipe");
var _counter_1 = require("./_counter");
var filter_1 = require("./filter");
function defaultTo(d) {
    return function (v) {
        return v == null ? d : v;
    };
}
test('should return last', function () {
    expect((0, first_1.first)([1, 2, 3])).toEqual(1);
});
test('empty array', function () {
    expect((0, first_1.first)([])).toEqual(undefined);
});
describe('pipe', function () {
    test('as no-fn', function () {
        var counter = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([1, 2, 3, 4, 5, 6], counter.fn(), first_1.first, function (x) { return x; });
        expect(counter.count).toHaveBeenCalledTimes(1);
        expect(result).toEqual(1);
    });
    test('as fn', function () {
        var counter = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([1, 2, 3, 4, 5, 6], counter.fn(), (0, first_1.first)());
        expect(counter.count).toHaveBeenCalledTimes(1);
        expect(result).toEqual(1);
    });
    test('with filter', function () {
        var counter = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([1, 2, 4, 8, 16], counter.fn(), (0, filter_1.filter)(function (x) { return x > 3; }), (0, first_1.first)(), defaultTo(0), function (x) { return x + 1; });
        expect(counter.count).toHaveBeenCalledTimes(3);
        expect(result).toEqual(5);
    });
    test('empty array', function () {
        var counter = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([], counter.fn(), (0, first_1.first)());
        expect(counter.count).toHaveBeenCalledTimes(0);
        expect(result).toEqual(undefined);
    });
    test('2 x first()', function () {
        var counter = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([[1, 2, 3], [4, 5], [6]], counter.fn(), (0, first_1.first)(), defaultTo([]), (0, first_1.first)());
        expect(counter.count).toHaveBeenCalledTimes(1);
        expect(result).toEqual(1);
    });
    test('complex', function () {
        var counter1 = (0, _counter_1.createCounter)();
        var counter2 = (0, _counter_1.createCounter)();
        var result = (0, pipe_1.pipe)([[1, 2, 3], [1], [4, 5, 6, 7], [1, 2, 3, 4]], counter1.fn(), (0, filter_1.filter)(function (arr) { return arr.length === 4; }), (0, first_1.first)(), defaultTo([]), counter2.fn(), (0, filter_1.filter)(function (x) { return x % 2 === 1; }), (0, first_1.first)());
        expect(counter1.count).toHaveBeenCalledTimes(3);
        expect(counter2.count).toHaveBeenCalledTimes(2);
        expect(result).toEqual(5);
    });
});
