"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
var map_1 = require("./map");
var pipe_1 = require("./pipe");
var take_1 = require("./take");
var filter_1 = require("./filter");
describe('data_first', function () {
    it('map', function () {
        var result = (0, map_1.map)([1, 2, 3], function (x) { return x * 2; });
        expect(result).toEqual([2, 4, 6]);
    });
    it('map.indexed', function () {
        var result = map_1.map.indexed([0, 0, 0], function (x, i) { return i; });
        expect(result).toEqual([0, 1, 2]);
    });
});
describe('data_last', function () {
    it('map', function () {
        var result = (0, pipe_1.pipe)([1, 2, 3], (0, map_1.map)(function (x) { return x * 2; }));
        expect(result).toEqual([2, 4, 6]);
    });
    it('map.indexed', function () {
        var result = (0, pipe_1.pipe)([0, 0, 0], map_1.map.indexed(function (x, i) { return i; }));
        expect(result).toEqual([0, 1, 2]);
    });
});
describe('pipe', function () {
    it('with take', function () {
        var count = jest.fn();
        var result = (0, pipe_1.pipe)([1, 2, 3], (0, map_1.map)(function (x) {
            count();
            return x * 10;
        }), (0, take_1.take)(2));
        expect(count).toHaveBeenCalledTimes(2);
        expect(result).toEqual([10, 20]);
    });
    it('indexed', function () {
        var count = jest.fn();
        var result = (0, pipe_1.pipe)([0, 0, 0], map_1.map.indexed(function (x, i) {
            count();
            return i;
        }), (0, take_1.take)(2));
        expect(count).toHaveBeenCalledTimes(2);
        expect(result).toEqual([0, 1]);
    });
    it('indexed: check index and items', function () {
        var indexes1 = [];
        var indexes2 = [];
        var anyItems1 = [];
        var anyItems2 = [];
        var result = (0, pipe_1.pipe)([1, 2, 3, 4, 5], map_1.map.indexed(function (x, i, items) {
            anyItems1.push(__spreadArray([], items, true));
            indexes1.push(i);
            return x;
        }), (0, filter_1.filter)(function (x) { return x % 2 === 1; }), map_1.map.indexed(function (x, i, items) {
            anyItems2.push(__spreadArray([], items, true));
            indexes2.push(i);
            return x;
        }));
        expect(result).toEqual([1, 3, 5]);
        expect(indexes1).toEqual([0, 1, 2, 3, 4]);
        expect(indexes2).toEqual([0, 1, 2]);
        expect(anyItems1).toEqual([
            [1],
            [1, 2],
            [1, 2, 3],
            [1, 2, 3, 4],
            [1, 2, 3, 4, 5],
        ]);
        expect(anyItems2).toEqual([[1], [1, 3], [1, 3, 5]]);
    });
});
